import os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
import matplotlib.colors as mcolors
import matplotlib.gridspec as gridspec
import matplotlib.patches as mpatches
import usefulfunctions as uf

here = os.path.abspath(os.path.dirname(__file__))

# plot parameters
fontsize = 11
lfontsize = 11
fullpagewidth = 7
fmt = 'svg'
dpi = 150
linewidth = 1.5
alpha = 0.1
nlevels = 51
colourmap = plt.cm.inferno
ax_lw = 1

# load all the data into a dict
temp = '100K'
fignum = '3'
last_time = 1e5
N0s = [1e17, 1e18, 1e19]
data = {}
for N0 in N0s:
    data[N0] = {}
    # TRPL
    trpl = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_{1}cm3_TRPL.csv'.format(temp, str(N0).replace('+', ''))), header=0, index_col=0)
    trpl.columns = [int(x) for x in trpl.columns]
    data[N0]['TRPL'] = trpl.loc[:, trpl.columns[trpl.columns <= last_time]]
    # MCR-ALS spectra
    spectra = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_{1}cm3_MCR-ALS_spectra.csv'.format(temp, str(N0).replace('+', ''))), header=0, index_col=0)
    data[N0]['MCR-ALS spectra'] = spectra
    # MCR-ALS kinetics
    kinetics = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_{1}cm3_MCR-ALS_kinetics.csv'.format(temp, str(N0).replace('+', ''))), header=0, index_col=0)
    data[N0]['MCR-ALS kinetics'] = kinetics[kinetics.index <= last_time]
    
# load Yong data
#yong = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_Yong2017_1TT_50K.csv'), header=0, index_col=0, squeeze=True)
    
    
# create the plotting grid
fig = plt.figure(figsize=(7, 14))

outergrid = gridspec.GridSpec(nrows=1,
                              ncols=2,
                              wspace=0.07,
                              width_ratios=[2, 1],
                              figure=fig)

leftgrid = gridspec.GridSpecFromSubplotSpec(nrows=5,
                                            ncols=1,
                                            height_ratios=[0.05, 1, 1, 1, 1],
                                            subplot_spec=outergrid[0],
                                            hspace=0.1)

rightgrid = gridspec.GridSpecFromSubplotSpec(nrows=5,
                                             ncols=1,
                                             height_ratios=[0.05, 1, 1, 1, 1],
                                             subplot_spec=outergrid[1],
                                             hspace=0.1)

# set axes
cbax = plt.subplot(leftgrid[0])
cbdl = plt.subplot(rightgrid[0])
cbdl.axis('off')

axa1 = plt.subplot(leftgrid[1])
axa2 = plt.subplot(rightgrid[1])

axb1 = plt.subplot(leftgrid[2])
axb2 = plt.subplot(rightgrid[2])

axc1 = plt.subplot(leftgrid[3])
axc2 = plt.subplot(rightgrid[3])

axd1 = plt.subplot(leftgrid[4])
axd2 = plt.subplot(rightgrid[4])
axd2.axis('off')


# panel a1 - colourmap
ax = axa1
N0 = 1e17
trpl = data[N0]['TRPL']
trpl[trpl < 1e-7] = 1e-7
x = trpl.columns.values
y = trpl.index.values
X, Y = np.meshgrid(x, y)
Z = trpl.values
levels = np.logspace(-6, 0, nlevels)
cs = ax.contourf(X, Y, Z, levels, norm=mcolors.LogNorm(), cmap=colourmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
    #c.set_rasterized(True)
    
ax.set_xscale('log')
ax.set_rasterization_zorder(-10)
cbar = fig.colorbar(cs, cax=cbax, orientation='horizontal', ticks=np.logspace(-7, 0, 8), extendrect=False)
cbar.ax.invert_xaxis()
cbax.xaxis.set_ticks_position('top')
cbar.ax.tick_params(axis='x', which='major', labelsize=fontsize, width=ax_lw, length=6)
ax.set_ylim([545, 805])
ax.set_xlim([1, max(x)])
ax.set_ylabel('Wavelength (nm)', fontsize=fontsize)
#ax.set_xlabel('Time (ns)', fontsize=fontsize)
ax.set_xticklabels([])
cbar.ax.set_xlabel('PL intensity (norm.)', fontsize=fontsize, labelpad=-45)

ax.text(4000, 775, r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0))), color='w', fontsize=fontsize)
#ax.text(8000, 565, 'TTA', color='w', fontsize=fontsize)
ax.add_patch(mpatches.Rectangle((4000, 550), 40000, 210, facecolor='None', edgecolor='w', linestyle='--', linewidth=1.5))
ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)
ax = uf.linearticks(ax, ymajorsep=50, yminorsep=10)
ax = uf.logticks(ax, which='x')
ax.text(0.03, 0.97, 'a', fontsize=14, fontweight='bold', color='k', transform=ax.transAxes, va='top', fontname='Arial')

# panel a2 - spectra
ax = axa2
spectra = data[N0]['MCR-ALS spectra']
ax.plot(spectra['S1'].values, spectra.index.values, color='darkred', label=r'S$_1$', linewidth=linewidth)
ax.fill_between(spectra['S1'].values, 805, spectra.index.values, color='darkred', alpha=0.1, linewidth=0)
ax.plot(spectra['TT'].values, spectra.index.values, color='blue', label=r'$^1$(TT)', linewidth=linewidth)
ax.fill_between(spectra['TT'].values, 805, spectra.index.values, color='blue', alpha=0.2, linewidth=0) 
ax.set_ylim([545, 805])
ax.set_xlim([0, 1.15])
#ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
#ax.set_ylabel('Extracted spectra (norm.)', fontsize=fontsize)
ax.set_yticklabels([])
ax.set_xticks([])
ax.legend(loc='upper right', frameon=False, fontsize=lfontsize, borderpad=0)
ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)
ax = uf.linearticks(ax, ymajorsep=50, yminorsep=10)

ax.set_title('MCR-ALS spectra', pad=10, fontsize=fontsize)

# panel b1 - colourmap
ax = axb1
N0 = 1e18
trpl = data[N0]['TRPL']
trpl[trpl < 1e-7] = 1e-7
x = trpl.columns.values
y = trpl.index.values
X, Y = np.meshgrid(x, y)
Z = trpl.values
levels = np.logspace(-7, 0, nlevels)
cs = ax.contourf(X, Y, Z, levels, norm=mcolors.LogNorm(), cmap=colourmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
    #c.set_rasterized(True)
    
ax.set_xscale('log')
ax.set_rasterization_zorder(-10)
#cbar = fig.colorbar(cs, cax=cbax, orientation='horizontal', ticks=np.logspace(-7, 0, 8))
#cbar.ax.invert_xaxis()
#cbax.xaxis.set_ticks_position('top')
ax.set_ylim([545, 805])
ax.set_xlim([1, max(x)])
ax.set_ylabel('Wavelength (nm)', fontsize=fontsize)
#ax.set_xlabel('Time (ns)', fontsize=fontsize)
ax.set_xticklabels([])

ax.text(4000, 775, r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0))), color='w', fontsize=fontsize)
#ax.text(8000, 565, 'TTA', color='w', fontsize=fontsize)
ax.add_patch(mpatches.Rectangle((4000, 550), 40000, 210, facecolor='None', edgecolor='w', linestyle='--', linewidth=1.5))
ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)
ax = uf.linearticks(ax, ymajorsep=50, yminorsep=10)
ax = uf.logticks(ax, which='x')
ax.text(0.03, 0.97, 'b', fontsize=14, fontweight='bold', color='k', transform=ax.transAxes, va='top', fontname='Arial')

# panel b2 - spectra
ax = axb2
spectra = data[N0]['MCR-ALS spectra']
ax.plot(spectra['S1'].values, spectra.index.values, color='darkred', label=r'S$_1$', linewidth=linewidth)
ax.fill_between(spectra['S1'].values, 805, spectra.index.values, color='darkred', alpha=0.1, linewidth=0)
ax.plot(spectra['TT'].values, spectra.index.values, color='blue', label=r'$^1$(TT)', linewidth=linewidth)
ax.fill_between(spectra['TT'].values, 805, spectra.index.values, color='blue', alpha=0.2, linewidth=0) 
ax.set_ylim([545, 805])
ax.set_xlim([0, 1.15])
#ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
#ax.set_ylabel('Extracted spectra (norm.)', fontsize=fontsize)
ax.set_yticklabels([])
ax.set_xticks([])
ax.legend(loc='upper right', frameon=False, fontsize=lfontsize, borderpad=0)
ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)
ax = uf.linearticks(ax, ymajorsep=50, yminorsep=10)

# panel c1 - colourmap
ax = axc1
N0 = 1e19
trpl = data[N0]['TRPL']
trpl[trpl < 1e-7] = 1e-7
x = trpl.columns.values
y = trpl.index.values
X, Y = np.meshgrid(x, y)
Z = trpl.values
levels = np.logspace(-7, 0, nlevels)
cs = ax.contourf(X, Y, Z, levels, norm=mcolors.LogNorm(), cmap=colourmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
    #c.set_rasterized(True)
    
ax.set_xscale('log')
ax.set_rasterization_zorder(-10)
#cbar = fig.colorbar(cs, cax=cbax, orientation='horizontal', ticks=np.logspace(-7, 0, 8))
#cbar.ax.invert_xaxis()
#cbax.xaxis.set_ticks_position('top')
ax.set_ylim([545, 805])
ax.set_xlim([1, max(x)])
ax.set_ylabel('Wavelength (nm)', fontsize=fontsize)
#ax.set_xlabel('Time (ns)', fontsize=fontsize)
ax.set_xticklabels([])

ax.text(4000, 775, r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0))), color='w', fontsize=fontsize)
#ax.text(8000, 565, 'TTA', color='w', fontsize=fontsize)
ax.add_patch(mpatches.Rectangle((4000, 550), 40000, 210, facecolor='None', edgecolor='w', linestyle='--', linewidth=1.5))
ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)
ax = uf.linearticks(ax, ymajorsep=50, yminorsep=10)
ax = uf.logticks(ax, which='x')
ax.text(0.03, 0.97, 'c', fontsize=14, fontweight='bold', color='k', transform=ax.transAxes, va='top', fontname='Arial')

# panel c2 - spectra
ax = axc2
spectra = data[N0]['MCR-ALS spectra']
ax.plot(spectra['S1'].values, spectra.index.values, color='darkred', label=r'S$_1$', linewidth=linewidth)
ax.fill_between(spectra['S1'].values, 805, spectra.index.values, color='darkred', alpha=0.1, linewidth=0)
ax.plot(spectra['TT'].values, spectra.index.values, color='blue', label=r'$^1$(TT)', linewidth=linewidth)
ax.fill_between(spectra['TT'].values, 805, spectra.index.values, color='blue', alpha=0.2, linewidth=0) 
ax.set_ylim([545, 805])
ax.set_xlim([0, 1.15])
#ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
#ax.set_ylabel('Extracted spectra (norm.)', fontsize=fontsize)
ax.set_yticklabels([])
ax.set_xticks([])
ax.legend(loc='upper right', frameon=False, fontsize=lfontsize, borderpad=0)
ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)
ax = uf.linearticks(ax, ymajorsep=50, yminorsep=10)

# panel d1 - kinetics
alphas = [0.3, 0.65, 1.0]
markers = ['o', 's', '^']
ax = axd1
for i, N0 in enumerate(N0s):
    kinetics = data[N0]['MCR-ALS kinetics']
    label = r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0)))
    ax.loglog(kinetics.index.values, kinetics['TT'].values, markersize=6, alpha=alphas[i], marker=markers[i], color='b', linestyle='None', mfc='None', markeredgewidth=1, label=label)
    #ax.loglog(kinetics.index.values, kinetics['TT'].values, 'b-', alpha=0.3, label='_nolegend_')

#ax.loglog(yong.index.values, yong.values, 'c-', linewidth=3, alpha=0.5, zorder=5, label='Ref. 18')

x = np.logspace(np.log10(4), np.log10(300), 1000)
ax.loglog(x, np.exp(-(x-4)/25), 'r--', linewidth=1, label=r'$\tau=25$ns', zorder=6)

ax = uf.logticks(ax)
ax.set_xlim([1, 5e4])
ax.set_ylim([1e-7, 2])
ax.set_xlabel('Time (ns)', fontsize=fontsize)
ax.set_ylabel(r'$^1$(TT) PL intensity (norm.)', fontsize=fontsize)

handles, labels = ax.get_legend_handles_labels()
leg1 = ax.legend(handles[0:3], labels[0:3], loc=(0.05, 0.18), fontsize=lfontsize, frameon=False, borderpad=0, handletextpad=0, handlelength=1.0)
ax.add_artist(leg1)
leg2 = ax.legend(handles[3:], labels[3:], loc=(0.47, 0.75), fontsize=lfontsize, frameon=False, borderpad=0, handletextpad=0.6, handlelength=1.5)


ax.axvline(100, color='k', alpha=0.3, linewidth=linewidth)
ax.axvline(4000, color='k', alpha=0.3, linewidth=linewidth)

ax.fill_between([4000, 1e5], 1e-7, 2, color='b', alpha=0.1, linewidth=0)

ax.annotate('', xy=(12000, 9e-7), xytext=(30000, 1e-4), arrowprops=dict(arrowstyle='<-'), fontsize=lfontsize)
ax.text(1.1e4, 1.5e-4, 'TTA', fontsize=lfontsize)

ax.text(4, 1.7e-7, 'region I', fontsize=12)
ax.text(200, 1.7e-7, 'region II', fontsize=12)
ax.text(4.7e3, 1.7e-7, 'region III', fontsize=12)
ax.text(0.03, 0.85, 'd', fontsize=14, fontweight='bold', color='k', transform=ax.transAxes, va='top', fontname='Arial')
ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)


fig.savefig('fig3_all.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')
