import os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
import matplotlib.gridspec as gridspec
from matplotlib.legend_handler import HandlerTuple
import usefulfunctions as uf

here = os.path.abspath(os.path.dirname(__file__))

# plot parameters
fontsize = 11
lfontsize = 11
fullpagewidth = 7
fmt = 'svg'
dpi = 150
linewidth = 1.5
alpha = 0.1
ax_lw = 1


# load data for panel B - diF-TES-ADT TT kinetics and simulation
dt_k = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_MCR-ALS_TTkinetics_100K.csv'), header=0, index_col=0)

TTsim = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100K_TTmodel.csv'), header=0, index_col=0)
T_Tsim = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100K_T_Tmodel.csv'), header=0, index_col=0)
T1sim = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100K_T1model.csv'), header=0, index_col=0)

# load data for panel C - magnetic field effect
mfe = pd.read_csv(os.path.join(here, 'data', 'mfe.csv'), header=0, index_col=0)
mfe_errors = pd.read_csv(os.path.join(here, 'data', 'mfe_errors.csv'), header=0, index_col=0)
mfe_sim_20_30 = pd.read_csv(os.path.join(here, 'data', 'simulations_20_30.csv'), header=None, index_col=0, squeeze=True)
mfe_sim_100_200 = pd.read_csv(os.path.join(here, 'data', 'simulations_100_200.csv'), header=None, index_col=0, squeeze=True)

# load data for spectra for panel D
extspe = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_total_spectra_250K_1e17cm3.csv'), header=0, index_col=0, squeeze=True)
exspe = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_excimer_spectra_250K_1e17cm3.csv'), header=0, index_col=0, squeeze=True)





# create the plotting grid
fig = plt.figure(figsize=(14, 10))

outergrid = gridspec.GridSpec(
    nrows=1,
    ncols=2,
    wspace=0.2,
    figure=fig)

leftgrid = gridspec.GridSpecFromSubplotSpec(
    nrows=2,
    ncols=1,
    height_ratios=[2, 1],
    subplot_spec=outergrid[0],
    hspace=0.22)

rightgrid = gridspec.GridSpecFromSubplotSpec(
    nrows=2,
    ncols=1,
    height_ratios=[1.25, 1],
    subplot_spec=outergrid[1],
    hspace=0.22)

# set axes
axb = plt.subplot(rightgrid[0])
axc = plt.subplot(leftgrid[1])
axd = plt.subplot(rightgrid[1])

# remove parts of plot that will be blank
plt.subplot(leftgrid[0]).axis('off')


# panel B
ax = axb
alphas = [0.3, 0.65, 1.0]
markers = ['o', 's', '^']
linestyles = [':', '--', '-']
for i, N0 in enumerate([1e17, 1e18, 1e19]):
    ttdata = dt_k[str(N0)]
    ttmodel = TTsim[str(N0)]
    t_tmodel = T_Tsim[str(N0)]
    t1model = T1sim[str(N0)]

    label = r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0)))
    
    ax.loglog(ttdata.index.values, ttdata.values, markersize=6, alpha=alphas[i], marker=markers[i], color='b', linestyle='None', mfc='None', markeredgewidth=1, label=label)
    ax.loglog(ttmodel.index.values, ttmodel, color='r', linestyle=linestyles[i], linewidth=linewidth, label=label, alpha=0.75)
    
    ax.loglog(ttmodel.index.values, t_tmodel, color='seagreen', linestyle=linestyles[i], linewidth=linewidth, label=label, alpha=0.75)
    ax.loglog(ttmodel.index.values, t1model, color='purple', linestyle=linestyles[i], linewidth=linewidth, label=label, alpha=0.75)
    
    ax.set_xlim([1, 1e5])
    ax.set_ylim([1e-7, 2])
    # ax.set_xlabel('Time (ns)', fontsize=fontsize)
    # ax.set_ylabel(r'$^1$(TT) population (norm.)', fontsize=fontsize)

# legends   
ha, la = ax.get_legend_handles_labels()
legend1 = ax.legend(handles=zip(ha[::4], ha[1::4]), labels=la[::4], numpoints=1, handler_map={tuple: HandlerTuple(ndivide=None)}, loc=(0.12, 0.09), fontsize=lfontsize, frameon=False, handletextpad=0.5)
ax.add_artist(legend1)
legend2 = ax.legend(handles=ha[-3:], labels=[r'$^1$(TT)', r'(T..T)', r'T$_1$'], loc=(0.46, 0.09), fontsize=lfontsize, edgecolor='None', framealpha=0.8, handlelength=1.5, handletextpad=0.5)

# ticks
ax = uf.logticks(ax)

# regions
ax.axvline(100, color='k', alpha=0.3, linewidth=linewidth)
ax.axvline(4000, color='k', alpha=0.3, linewidth=linewidth)

ax.text(4, 1.7e-7, 'region I', fontsize=lfontsize)
ax.text(200, 1.7e-7, 'region II', fontsize=lfontsize)
ax.text(6e3, 1.7e-7, 'region III', fontsize=lfontsize)

# axes labels and limits
ax.set_xlim([1, 1e5])
ax.set_ylim([1e-7, 2])
ax.set_xlabel('Time (ns)', fontsize=fontsize)
ax.set_ylabel(r'$^1$(TT) PL intensity (norm.)', fontsize=fontsize)

ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)


# panel C
color1 = 'dodgerblue'
color2 = 'darkorange'
ax = axc
ax.errorbar(mfe.index.values, 100*mfe['20-30ns'].values, yerr=100*mfe_errors['20-30ns'].values, capsize=5, color=color1, marker='o', linestyle='None', markersize=6, mew=1, label='20-30ns')
ax.plot(1000*mfe_sim_20_30.index.values, 100*mfe_sim_20_30.values, color=color1, linestyle='-', linewidth=linewidth, label='_nolegend_')

ax.errorbar(mfe.index.values, 100*mfe['100-200ns'].values, yerr=100*mfe_errors['100-200ns'].values, capsize=5, color=color2, marker='s', linestyle='None', markersize=6, mew=1, label='100-200ns')
ax.plot(1000*mfe_sim_100_200.index.values, 100*mfe_sim_100_200.values, color=color2, linestyle='--', linewidth=linewidth, label='_nolegend_')

ax.axhline(0, color='k', alpha=0.5, linewidth=0.5)

ax.set_xlim([0, 260])

ax.set_xlabel('Magnetic field strength (mT)', fontsize=fontsize)
ax.set_ylabel(r'$\Delta$PL/PL (%)', fontsize=fontsize, labelpad=-12)

ax.legend(frameon=False, fontsize=lfontsize, loc=(0.28, 0.75), handlelength=1.5, handletextpad=0)

#ax.text(100, 9.5, r'simulation', fontsize=lfontsize, color=color1)
#ax.text(170, -6, r'simulation', fontsize=lfontsize, color=color2)

ax.text(180, 12, 'lines: simulation', fontsize=lfontsize)

ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=10, yminorsep=2)


# panel D
ax = axd

colours = [plt.cm.tab10(0), plt.cm.tab10(2), plt.cm.tab10(6)]
linestyles = ['-', ':', '--']
times = [4, 20, 100]
alphas = [0.9, 0.6, 0.6]

for i, time in enumerate(times):
    ax.plot(extspe.index.values, extspe[str(time)].values, color=colours[i], linestyle='-', alpha=alphas[i], linewidth=linewidth, label='{0}ns'.format(time))
    #smoothed = extspe[str(time)].rolling(61, center=True).mean()
    #ax.plot(smoothed.index.values, smoothed.values, color=colours[i], linestyle='-', alpha=1, linewidth=1, label='{0}ns'.format(time))

ax.plot(exspe.index.values, exspe.values, 'k-', alpha=0.2)
ax.plot(exspe.index.values, exspe.rolling(61, center=True).mean(), 'k:', alpha=1, linewidth=linewidth, label='Excimer')

ax.set_xlim([545, 805])
ax.set_ylim([0, 3.2])
    
ax.set_ylabel('PL intensity (norm.)', fontsize=fontsize)
ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)

handles, labels = ax.get_legend_handles_labels()
leg1 = ax.legend(handles=handles[0:-1], labels=labels[0:-1], frameon=False, fontsize=lfontsize, loc='upper left', borderpad=0.6, handlelength=1.5, )
for lh in leg1.legendHandles: 
    lh.set_alpha(1)
ax.add_artist(leg1)
leg2 = ax.legend(handles=[handles[-1]], labels=[labels[-1]], frameon=False, fontsize=lfontsize, loc='upper right', borderpad=0.6, handlelength=1.5, )

ax.yaxis.set_major_locator(mticker.MultipleLocator(0.5))
ax = uf.format_axes(ax, labelsize=fontsize, linewidth=ax_lw)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.5, yminorsep=0.25)


fig.savefig('fig4_bcd.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')
