import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf


here = os.path.abspath(os.path.dirname(__file__))


# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 300
linewidth = 1.5
alpha = 0.1


# load the kinetics
soln_kin_vis = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100uM_toluene_14uJcm2_kinetics_vis.csv'), header=0, index_col=0)
soln_kin_nir = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100uM_toluene_14uJcm2_kinetics_nir.csv'), header=0, index_col=0)
film_kin_vis = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_film_35uJcm2_kinetics_vis.csv'), header=0, index_col=0)
film_kin_nir = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_film_35uJcm2_kinetics_nir.csv'), header=0, index_col=0)

# load the spectra
soln_spectra = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100uM_toluene_14uJcm2_spectra.csv'), header=0, index_col=0)
film_spectra = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_film_35uJcm2_spectra.csv'), header=0, index_col=0)
    
    
# create the grid
fig, axes = plt.subplots(nrows=2,
                         ncols=2,
                         gridspec_kw={'hspace': 0.3,
                                      'wspace': 0.2,
                                      'width_ratios': [2, 1]},
                         figsize=(14, 8))


# panel A - solution spectra
ax = axes[0, 0]
colours = ['darkblue', 'seagreen', 'darkorange', 'darkred']
for i, time in enumerate(soln_spectra.columns):
    ax.plot(soln_spectra.index.values, 1000*soln_spectra[time].values, color=colours[i], linewidth=linewidth, label=time)
ax.legend(frameon=False, fontsize=lfontsize, loc=(0.5, 0.47))
ax.axhline(0, linewidth=0.5, alpha=0.5)
ax.set_xlim([360, 1290])
ax.set_xticks(np.linspace(400, 1200, 9))
ax.set_ylabel(r'$\Delta$OD ($\times$10$^{{-3}}$)', fontsize=fontsize)
ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)

ax = uf.linearticks(ax, xmajorsep=100, xminorsep=50, ymajorsep=0.2, yminorsep=0.1)
ax = uf.format_axes(ax, labelsize=fontsize)


# panel B - solution kinetics
ax = axes[0, 1]
wavelengths = ['430nm']
colours = ['blue']
for i in [0]:
    axes[0, 0].axvline(float(wavelengths[i].rstrip('nm')), color=colours[i], alpha=0.25, linewidth=4)
    ax.semilogx(soln_kin_vis.index.values, soln_kin_vis[wavelengths[i]].values, color=colours[i], linewidth=linewidth, alpha=0.8, label=wavelengths[i])

t = np.logspace(-1, 4, 1000)
ax.plot(t, np.exp(-0.0000833*t)+0.05, 'r--', linewidth=2, label=r'$\tau=12$ns', alpha=0.3)

ax.set_xlim([0.5, 5e3])
ax.set_ylim([0, 1.3])
ax.set_ylabel(r'$\Delta$OD (norm.)', fontsize=fontsize)
ax.set_xlabel('Time (ps)', fontsize=fontsize)
ax.legend(frameon=False, fontsize=lfontsize, loc='lower left', borderpad=0)

ax = uf.linearticks(ax, ymajorsep=0.2, yminorsep=0.1)
ax = uf.logticks(ax, which='x')
ax = uf.format_axes(ax, labelsize=fontsize)


# panel C - film spectra
ax = axes[1, 0]
colours = ['darkblue', 'seagreen', 'darkorange', 'darkred']
for i, time in enumerate(film_spectra.columns):
    ax.plot(film_spectra.index.values, 1000*film_spectra[time].values, color=colours[i], linewidth=linewidth, label=time)
ax.legend(frameon=False, fontsize=lfontsize, loc=(0.5, 0.47))
ax.axhline(0, linewidth=0.5, alpha=0.5)
ax.set_xlim([360, 1290])
ax.set_xticks(np.linspace(400, 1200, 9))
ax.set_ylabel(r'$\Delta$OD ($\times$10$^{{-3}}$)', fontsize=fontsize)
ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)

ax = uf.linearticks(ax, xmajorsep=100, xminorsep=50, ymajorsep=0.2, yminorsep=0.1)
ax = uf.format_axes(ax, labelsize=fontsize)


# panel B - solution kinetics
ax = axes[1, 1]
wavelengths = ['380nm', '430nm', '495nm']
colours = ['darkviolet', 'blue', 'springgreen']
for i in [0, 1, 2]:
    axes[1, 0].axvline(float(wavelengths[i].rstrip('nm')), color=colours[i], alpha=0.25, linewidth=4)
    ax.semilogx(film_kin_vis.index.values, film_kin_vis[wavelengths[i]].values, color=colours[i], linewidth=linewidth, alpha=0.8, label=wavelengths[i])

t = np.logspace(-1, 4, 1000)
ax.plot(t, 0.95*np.exp(-0.01*t), 'r--', linewidth=2, label=r'$\tau=100$ps', alpha=0.3)

ax.set_xlim([0.5, 5e3])
ax.set_ylim([0, 1.1])
ax.set_ylabel(r'$\Delta$OD (norm.)', fontsize=fontsize)
ax.set_xlabel('Time (ps)', fontsize=fontsize)
ax.legend(frameon=False, fontsize=lfontsize, loc='lower left', borderpad=0)

ax = uf.linearticks(ax, ymajorsep=0.2, yminorsep=0.1)
ax = uf.logticks(ax, which='x')
ax = uf.format_axes(ax, labelsize=fontsize)

fig.savefig('figS13_abcd.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')

