import os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
import matplotlib.gridspec as gridspec
from matplotlib.legend_handler import HandlerTuple
import usefulfunctions as uf

here = os.path.abspath(os.path.dirname(__file__))


# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 300
linewidth = 1.5
alpha = 0.1


 # load the data into a dict
temperatures = ['77K', '100K', '150K', '200K', '250K', '291K']
data = {}
for temp in temperatures:
    data[temp] = {}
    datas = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_MCR-ALS_TT.csv'.format(temp)), header=0, index_col=0)
    data[temp]['datas'] = datas
    model = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_model.csv'.format(temp)), header=0, index_col=0)
    data[temp]['model'] = model

rates = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_rates.csv'.format(temp)), header=0, index_col=0)
rates.columns = [1000/int(x.replace('K', '')) for x in rates.columns]

rateslb = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_rates_lowerbounds.csv'), header=0, index_col=0)
rateslb.columns = [1000/int(x.replace('K', '')) for x in rateslb.columns]

ratesub = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_rates_upperbounds.csv'), header=0, index_col=0)
ratesub.columns = [1000/int(x.replace('K', '')) for x in ratesub.columns]


# create the plotting grid
fig = plt.figure(figsize=(14, 10.5))

outergrid = gridspec.GridSpec(nrows=2,
                              ncols=1,
                              hspace=0.27,
                              height_ratios=[3, 1],
                              figure=fig)

topgrid = gridspec.GridSpecFromSubplotSpec(nrows=2,
                                           ncols=3,
                                           subplot_spec=outergrid[0],
                                           hspace=0.05,
                                           wspace=0.05)

bottomgrid = gridspec.GridSpecFromSubplotSpec(nrows=1,
                                              ncols=5,
                                              subplot_spec=outergrid[1],
                                              width_ratios=[1, 1, 1, 0.4, 1],
                                              wspace=0.08)

axa = plt.subplot(topgrid[0, 0])
axb = plt.subplot(topgrid[0, 1])
axc = plt.subplot(topgrid[0, 2])
axd = plt.subplot(topgrid[1, 0])
axe = plt.subplot(topgrid[1, 1])
axf = plt.subplot(topgrid[1, 2])
axes = [axa, axb, axc, axd, axe, axf]

axg = plt.subplot(bottomgrid[0])
axh = plt.subplot(bottomgrid[1])
axi = plt.subplot(bottomgrid[2])
plt.subplot(bottomgrid[3]).axis('off')
axj = plt.subplot(bottomgrid[4])


# plot the data and model in the top grid
alphas = [0.3, 0.65, 1.0]
markers = ['o', 's', '^']
linestyles = [':', '--', '-']

for ii, temp in enumerate(temperatures):
    
    datas = data[temp]['datas']
    model = data[temp]['model']
    
    for i, N0 in enumerate([1e17, 1e18, 1e19]):
        
        ttdata = datas[str(N0)]
        ttmodel = model[str(N0)]
        ax = axes[ii]
        label = r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0)))
        
        ax.loglog(ttdata.index.values, ttdata.values, markersize=6, alpha=alphas[i], marker=markers[i], color='b', linestyle='None', mfc='None', markeredgewidth=1, label=label)
        ax.loglog(ttmodel.index.values, ttmodel, color='r', linestyle=linestyles[i], linewidth=1, label=label, alpha=0.75)
        ax.set_xlim([1, 9e4])
        ax.set_ylim([1e-7, 2])
        
    ha, la = ax.get_legend_handles_labels()
    ax.legend(handles=zip(ha[::2], ha[1::2]), labels=la[::2], numpoints=1, handler_map={tuple: HandlerTuple(ndivide=None)}, frameon=False, loc=(0.01, 0.01), fontsize=lfontsize, borderpad=0, handletextpad=0.5, handlelength=1.5)
    ax = uf.logticks(ax)
    ax = uf.format_axes(ax, labelsize=fontsize)
    ax.text(0.97, 0.97, temp, fontsize=fontsize, transform=ax.transAxes, va='top', ha='right')
    
    if ii in [0, 3]:
        ax.set_ylabel('PL Intensity (norm.)', fontsize=fontsize)
    else:
        ax.set_yticklabels([])
    if ii in [3, 4, 5]:
        ax.set_xlabel('Time (ns)', fontsize=fontsize)
    else:
        ax.set_xticklabels([])

    
# plot the rates in the bottom grid
kHOP = rates.loc['kHOP', :]
kHOPlb = rateslb.loc['kHOP', :]
kHOPub = ratesub.loc['kHOP', :]
kHOPerr = np.vstack((kHOP-kHOPlb, kHOPub-kHOP))

k_HOP = rates.loc['k_HOP', :]
k_HOPlb = rateslb.loc['k_HOP', :]
k_HOPub = ratesub.loc['k_HOP', :]
k_HOPerr = np.vstack((k_HOP-k_HOPlb, k_HOPub-k_HOP))

kHOP2 = rates.loc['kHOP2', :]
kHOP2lb = rateslb.loc['kHOP2', :]
kHOP2ub = ratesub.loc['kHOP2', :]
kHOP2err = np.vstack((kHOP2-kHOP2lb, kHOP2ub-kHOP2))

kTTA = rates.loc['kTTA', :]
kTTAlb = rateslb.loc['kTTA', :]
kTTAub = ratesub.loc['kTTA', :]
kTTAerr = np.vstack((kTTA-kTTAlb, kTTAub-kTTA))

kTTNR = rates.loc['kTTNR', :]
kTTNRlb = rateslb.loc['kTTNR', :]
kTTNRub = ratesub.loc['kTTNR', :]
kTTNRerr = np.vstack((kTTNR-kTTNRlb, kTTNRub-kTTNR))

kTNR = rates.loc['kTNR', :]
kTNRlb = rateslb.loc['kTNR', :]
kTNRub = ratesub.loc['kTNR', :]
kTNRerr = np.vstack((kTNR-kTNRlb, kTNRub))

#kTTAfromkHOP = rates.loc['kTTAfromkHOP', :]
#kTTAfromkHOPlb = rateslb.loc['kTTAfromkHOP', :]
#kTTAfromkHOPub = ratesub.loc['kTTAfromkHOP', :]
#kTTAfromkHOPerr = np.vstack((kTTAfromkHOP-kTTAfromkHOPlb, kTTAfromkHOPub-kTTAfromkHOP))

kRELAX = rates.loc['kRELAX', :]
kRELAXlb = rateslb.loc['kRELAX', :]
kRELAXub = ratesub.loc['kRELAX', :]
kRELAXerr = np.vstack((kRELAX-kRELAXlb, kRELAXub-kRELAX))


ms = 7
ls = '-'
al = 0.6
cs = 4

ax = axg
ax.errorbar(kHOP.index.values, kHOP.values, yerr=kHOPerr, capsize=cs, marker='None', markersize=ms, mew=1, alpha=al, linestyle=ls, color='teal', label=r'$k_{{hop}}$')
ax.errorbar(k_HOP.index.values, k_HOP.values, yerr=k_HOPerr, capsize=cs, marker='None', markersize=ms, mew=1, alpha=al, linestyle='--', color='blue', label=r'$k_{{-hop}}$')
ax.errorbar(kHOP2.index.values, kHOP2.values, yerr=kHOP2err, capsize=cs, marker='None', markersize=ms, mew=1, alpha=al, linestyle=':', color='darkviolet', label=r'$k_{{hop2}}$')
ha, la = ax.get_legend_handles_labels()
ax.legend([h[0] for h in ha], la, frameon=False, fontsize=lfontsize, loc='lower left', borderpad=0, labelspacing=0.1, handlelength=1.5)
ax.set_ylabel(r'rate (ns$^{{-1}}$)', fontsize=fontsize)
ax.set_xlabel(r'1000/T (K$^{{-1}}$)', fontsize=fontsize)
ax.set_xticks([4, 6, 8, 10, 12])
ax.set_yscale('log')
ax.set_ylim([2e-6, 5e-1])
ax = uf.logticks(ax, which='y')
ax = uf.linearticks(ax, xmajorsep=2, xminorsep=1)
ax = uf.format_axes(ax, labelsize=fontsize)

ax = axh
ax.errorbar(kTTNR.index.values, kTTNR.values, yerr=kTTNRerr, capsize=cs, marker='None', markersize=ms, mew=1, alpha=al, linestyle=ls, color='firebrick', label=r'$k_{{ttnr}}$')
ax.errorbar(kTNR.index.values, kTNR.values, yerr=kTNRerr, capsize=cs, marker='None', markersize=ms, mew=1, alpha=al, linestyle='--', color='darkorange', label=r'$k_{{tnr}}$')
ha, la = ax.get_legend_handles_labels()
ax.legend([h[0] for h in ha], la, frameon=False, fontsize=lfontsize, loc='center right', borderpad=0, labelspacing=0.1, handlelength=1.5)
ax.set_xlabel(r'1000/T (K$^{{-1}}$)', fontsize=fontsize)
ax.set_xticks([4, 6, 8, 10, 12])
ax.set_yscale('log')
ax.set_ylim([2e-6, 5e-1])
ax.set_yticklabels([])
ax = uf.logticks(ax, which='y')
ax = uf.linearticks(ax, xmajorsep=2, xminorsep=1)
ax = uf.format_axes(ax, labelsize=fontsize)

ax = axi
ax.errorbar(kRELAX.index.values, kRELAX.values, yerr=kRELAXerr, capsize=cs, marker='None', markersize=ms, mew=1, alpha=al, linestyle=ls, color='black', label=r'$k_{{relax}}$')
ha, la = ax.get_legend_handles_labels()
ax.legend([h[0] for h in ha], la, frameon=False, fontsize=lfontsize, loc='upper right', borderpad=0, labelspacing=0.1, handlelength=1.5)
ax.set_xlabel(r'1000/T (K$^{{-1}}$)', fontsize=fontsize)
ax.set_xticks([4, 6, 8, 10, 12])
ax.set_yscale('log')
ax.set_ylim([2e-6, 5e-1])
ax.set_yticklabels([])
ax = uf.logticks(ax, which='y')
ax = uf.linearticks(ax, xmajorsep=2, xminorsep=1)
ax = uf.format_axes(ax, labelsize=fontsize)

ax = axj
ax.errorbar(kTTA.index.values, kTTA.values, yerr=kTTAerr, capsize=cs, marker='None', markersize=ms, mew=1, alpha=al, linestyle=ls, color='springgreen', label=r'$k_{{tta}}$')
#ax.errorbar(kTTAfromkHOP.index.values, kTTAfromkHOP.values, yerr=kTTAfromkHOPerr, capsize=cs, marker='None', markersize=ms, mew=1, alpha=al, linestyle='--', color='seagreen', label=r'$\frac{{2}}{{3}}\pi Vk_{{hop}}$')
ha, la = ax.get_legend_handles_labels()
ax.legend([h[0] for h in ha], la, frameon=False, fontsize=lfontsize, loc='lower left', borderpad=0, labelspacing=0.1, handlelength=1.5)
ax.set_ylabel(r'rate (cm$^{{3}}$ ns$^{{-1}}$)', fontsize=fontsize)
ax.set_xlabel(r'1000/T (K$^{{-1}}$)', fontsize=fontsize)
ax.set_xticks([4, 6, 8, 10, 12])
ax.set_yscale('log')
ax = uf.logticks(ax, which='y')
ax = uf.linearticks(ax, xmajorsep=2, xminorsep=1)
ax = uf.format_axes(ax, labelsize=fontsize)


fig.savefig('figS16_rest.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')