import os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
import matplotlib.gridspec as gridspec
from matplotlib.legend_handler import HandlerTuple
import usefulfunctions as uf

here = os.path.abspath(os.path.dirname(__file__))


# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 300
linewidth = 1.5
alpha = 0.1


 # load the data into a dict
temperatures = ['77K', '100K', '150K', '200K', '250K', '291K']
data = {}
for temp in temperatures:
    data[temp] = {}
    datas = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_MCR-ALS_TT.csv'.format(temp)), header=0, index_col=0)
    data[temp]['datas'] = datas
    model = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_model.csv'.format(temp)), header=0, index_col=0)
    data[temp]['model'] = model
rates = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_rates.csv'.format(temp)), header=0, index_col=0)
rates.columns = [1000/int(x.replace('K', '')) for x in rates.columns]


# create the plotting grid
fig = plt.figure(figsize=(14, 10.5))

outergrid = gridspec.GridSpec(nrows=2,
                              ncols=1,
                              hspace=0.27,
                              height_ratios=[3, 1],
                              figure=fig)

topgrid = gridspec.GridSpecFromSubplotSpec(nrows=2,
                                           ncols=3,
                                           subplot_spec=outergrid[0],
                                           hspace=0.05,
                                           wspace=0.05)

bottomgrid = gridspec.GridSpecFromSubplotSpec(nrows=1,
                                              ncols=3,
                                              subplot_spec=outergrid[1],
                                              wspace=0.5)

axa = plt.subplot(topgrid[0, 0])
axb = plt.subplot(topgrid[0, 1])
axc = plt.subplot(topgrid[0, 2])
axd = plt.subplot(topgrid[1, 0])
axe = plt.subplot(topgrid[1, 1])
axf = plt.subplot(topgrid[1, 2])
axes = [axa, axb, axc, axd, axe, axf]

axg = plt.subplot(bottomgrid[0])
axh = plt.subplot(bottomgrid[1])
axi = plt.subplot(bottomgrid[2])


# plot the data and model in the top grid
alphas = [0.3, 0.65, 1.0]
markers = ['o', 's', '^']
linestyles = [':', '--', '-']

for ii, temp in enumerate(temperatures):
    
    datas = data[temp]['datas']
    model = data[temp]['model']
    
    for i, N0 in enumerate([1e17, 1e18, 1e19]):
        
        ttdata = datas[str(N0)]
        ttmodel = model[str(N0)]
        ax = axes[ii]
        label = r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0)))
        
        ax.loglog(ttdata.index.values, ttdata.values, markersize=6, alpha=alphas[i], marker=markers[i], color='b', linestyle='None', mfc='None', markeredgewidth=1, label=label)
        ax.loglog(ttmodel.index.values, ttmodel, color='r', linestyle=linestyles[i], linewidth=1, label=label, alpha=0.75)
        ax.set_xlim([1, 9e4])
        ax.set_ylim([1e-7, 2])
        
    ha, la = ax.get_legend_handles_labels()    
    ax.legend(handles=zip(ha[::2], ha[1::2]), labels=la[::2], numpoints=1, handler_map={tuple: HandlerTuple(ndivide=None)}, frameon=False, loc=(0.01, 0.01), fontsize=lfontsize, borderpad=0, handletextpad=0.5, handlelength=1.5)
    ax = uf.logticks(ax)
    ax = uf.format_axes(ax, labelsize=fontsize)    
    ax.text(0.97, 0.97, temp, fontsize=fontsize, transform=ax.transAxes, va='top', ha='right')
    
    if ii in [0, 3]:
        ax.set_ylabel('PL Intensity (norm.)', fontsize=fontsize)
    else:
        ax.set_yticklabels([])
    if ii in [3, 4, 5]:
        ax.set_xlabel('Time (ns)', fontsize=fontsize)
    else:
        ax.set_xticklabels([])

    
# plot the rates in the bottom grid
kHOP = rates.loc['kHOP', :]
k_HOP = rates.loc['k_HOP', :]
kDECOH = rates.loc['kDECOH', :]
kTTA = rates.loc['kTTA', :]
kTTNR = rates.loc['kTTNR', :]
kTNR = rates.loc['kTNR', :]
#kTTAfromkHOP = rates.loc['kTTAfromkHOP', :]

ms = 7
ls = '-'
al = 0.35

axg.semilogy(kHOP.index.values, kHOP.values, marker='o', markersize=ms, markeredgewidth=0, alpha=al, linestyle=ls, color='teal', label=r'$k_{{hop}}$')
axg.semilogy(k_HOP.index.values, k_HOP.values, marker='s', markersize=ms, markeredgewidth=0, alpha=al, linestyle=ls, color='blue', label=r'$k_{{-hop}}$')
axg.semilogy(kDECOH.index.values, kDECOH.values, marker='^', markersize=ms, markeredgewidth=0, alpha=al, linestyle=ls, color='darkviolet', label=r'$k_{{hop2}}$')
axg.legend(frameon=False, fontsize=lfontsize, borderpad=0, labelspacing=0.1, handlelength=1.5, handletextpad=0.5, loc=(0.03, 0))
axg.set_ylabel(r'rate (ns$^{{-1}}$)', fontsize=fontsize)
axg.set_xlabel(r'1000/T (K$^{{-1}}$)', fontsize=fontsize)
axg.set_xticks([4, 6, 8, 10, 12])
axg = uf.logticks(axg, which='y')
axg = uf.linearticks(axg, xmajorsep=2, xminorsep=1)
axg = uf.format_axes(axg, labelsize=fontsize)

axh.semilogy(kTTNR.index.values, kTTNR.values, marker='o', markersize=ms, markeredgewidth=0, alpha=al, linestyle=ls, color='firebrick', label=r'$k_{{ttnr}}$')
axh.semilogy(kTNR.index.values, kTNR.values, marker='s', markersize=ms, markeredgewidth=0, alpha=al, linestyle=ls, color='darkorange', label=r'$k_{{tnr}}$')
axh.legend(frameon=False, fontsize=lfontsize, labelspacing=0.1, handlelength=1.5, handletextpad=0.5)
axh.set_ylabel(r'rate (ns$^{{-1}}$)', fontsize=fontsize)
axh.set_xlabel(r'1000/T (K$^{{-1}}$)', fontsize=fontsize)
axh = uf.logticks(axh, which='y')
axh = uf.linearticks(axh, xmajorsep=2, xminorsep=1)
axh = uf.format_axes(axh, labelsize=fontsize)

axi.semilogy(kTTA.index.values, kTTA.values, marker='o', markersize=ms, markeredgewidth=0, alpha=al, linestyle=ls, color='springgreen', label=r'$k_{{tta}}$')
#axi.semilogy(kTTAfromkHOP.index.values, kTTAfromkHOP.values, marker='s', markersize=ms, markeredgewidth=0, alpha=al, linestyle=ls, color='seagreen', label=r'$\frac{{2}}{{3}}\pi Vk_{{hop}}$')
axi.legend(frameon=False, fontsize=lfontsize, labelspacing=0.1, handlelength=1.5, handletextpad=0.5, loc=(0.45, 0.5))
axi.set_ylabel(r'rate (cm$^{{3}}$ ns$^{{-1}}$)', fontsize=fontsize)
axi.set_xlabel(r'1000/T (K$^{{-1}}$)', fontsize=fontsize)
axi = uf.logticks(axi, which='y')
axi = uf.linearticks(axi, xmajorsep=2, xminorsep=1)
axi = uf.format_axes(axi, labelsize=fontsize)



fig.savefig('figS17_rest.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')