import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
import usefulfunctions as uf


here = os.path.abspath(os.path.dirname(__file__))


# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 300
linewidth = 1.5
alpha = 0.1


# load the data
temperature = '150K'
rateslist = ['kSF', 'k_SF', 'kSNR', 'kTTNR', 'kTNR', 'kHOP', 'k_HOP', 'kHOP2', 'kTTA', 'kRELAX']
rateswitherrors = pd.read_csv(os.path.join(here, 'data', '{0}_rateswitherrors.csv'.format(temperature)), header=0, index_col=0)
costs = pd.read_csv(os.path.join(here, 'data', '{0}_costvalues.csv'.format(temperature)), header=None, index_col=0, squeeze=True)
mincost = costs['minimized cost']
costthreshold = costs['cost threshold']
costcurves = {}
for rate in rateslist:
    costcurve = pd.read_csv(os.path.join(here, 'data', '{0}_{1}.csv'.format(temperature, rate)), index_col=0, header=0, squeeze=True)
    costcurves[rate] = costcurve


# make the plotting grid
fig, axes = plt.subplots(nrows=2,
                         ncols=5,
                         sharey=False,
                         gridspec_kw={'hspace': 0.65,
                                      'wspace': 0.1},
                         figsize=(14, 6))


# iterate through the rates and populate each panel
for i, rate in enumerate(rateslist):
    
    ax = axes[int(i/5), i%5]
    
    ratevalue = rateswitherrors.loc[rate, 'value']
    ratelower = rateswitherrors.loc[rate, 'lowerbound']
    rateupper = rateswitherrors.loc[rate, 'upperbound']
    
    costcurve = costcurves[rate]
    
    ax.axvline(ratevalue, color='0.5', linewidth=linewidth)
    ax.axhline(costthreshold, color='0.5', linewidth=linewidth)
    ax.axhline(mincost, color='0.5', linewidth=linewidth, linestyle=':')
    
    ax.semilogx(costcurve.index.values, costcurve.values, 'k-', linewidth=linewidth)
    ax.fill_between(costcurve.index.values, costcurve.values, costthreshold, color='r', alpha=0.1, where=costcurve.values<costthreshold, interpolate=True)
    
    ax.semilogx([ratelower, rateupper], [costthreshold, costthreshold], 'ro', mfc='None')
    
    ax.set_ylim([mincost*0.9, 1.1*costthreshold])
    ax.set_xlim([costcurve.index.values[0], costcurve.index.values[-1]])
    
    ax.set_xlabel(r'rate ({0}ns$^{{-1}}$)'.format('cm$^{{3}}$' if rate == 'kTTA' else ''), fontsize=fontsize)
    
    ax.text(0.99, 0.07, r'$k_{{{0}}}$'.format(rate[1:].lower().replace('_', '-')), fontsize=fontsize, transform=ax.transAxes, ha='right')
    
    if i in [0, 5]:
        ax.set_ylabel(r'cost parameter, $\chi$', fontsize=fontsize)
    else:
        ax.set_yticklabels([])

    ax = uf.logticks(ax, which='x')
    ax = uf.linearticks(ax, ymajorsep=0.005, yminorsep=0.001)
    
    if i < 3:
        ax.set_facecolor('0.9')
        
    ax = uf.format_axes(ax, labelsize=fontsize)
    ax.tick_params(axis='x', which='major', labelrotation=30)

fig.savefig('figS20_all.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')