import os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
from matplotlib.legend_handler import HandlerTuple
import usefulfunctions as uf

here = os.path.abspath(os.path.dirname(__file__))


# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 300
linewidth = 1.5
alpha = 0.1


# load data
ttdatas = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100K_MCR-ALS_TT.csv'), header=0, index_col=0)
ttmodels = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100K_Dovermodel_TT.csv'), header=0, index_col=0)
s1models = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100K_Dovermodel_S1.csv'), header=0, index_col=0)
S1_spectrum = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100K_Dovermodel_S1_20us_1e19cm3.csv'), header=0, index_col=0, squeeze=True)
measured_spectrum = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_100K_measured_spectrum_20us_1e19cm3.csv'), header=0, index_col=0, squeeze=True)


# create the plotting grid
fig, axes = plt.subplots(nrows=1, ncols=2, gridspec_kw={'wspace': 0.25}, figsize=(14, 4))


# plot the data and model kinetics in the LH pane
ax = axes[0]

alphas = [0.3, 0.65, 1.0]
markers = ['o', 's', '^']
linestyles = [':', '--', '-']

for i, N0 in enumerate([1e17, 1e18, 1e19]):
    
    ttdata = ttdatas[str(N0)]
    ttmodel = ttmodels[str(N0)]
    s1model = s1models[str(N0)]
    label = r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0)))
    
    ax.loglog(ttdata.index.values, ttdata.values, markersize=6, alpha=alphas[i], marker=markers[i], color='b', linestyle='None', mfc='None', markeredgewidth=1, label=label)
    ax.loglog(ttmodel.index.values, ttmodel, color='r', linestyle=linestyles[i], linewidth=linewidth, label=label, alpha=0.75)
    
    ax.loglog(s1model.index.values, s1model, color='k', linestyle=linestyles[i], linewidth=linewidth, label='_nolegend_', alpha=1)
    
    ax.set_xlim([1, 1e5])
    ax.set_ylim([1e-7, 2])
    ax.set_xlabel('Time (ns)', fontsize=fontsize)
    ax.set_ylabel('PL intensity (norm.)', fontsize=fontsize)
    
ha, la = ax.get_legend_handles_labels()    
ax.legend(handles=zip(ha[::2], ha[1::2]), labels=la[::2], numpoints=1, handler_map={tuple: HandlerTuple(ndivide=None)}, frameon=False, loc='upper right', fontsize=lfontsize)

ax.text(5e2, 8e-6, r'S$_1$', fontsize=fontsize, color='k')
ax.text(1.7e4, 4e-5, r'$^1$(TT)', fontsize=fontsize, color='b')
ax.text(1.3, 2e-7, '100K', fontsize=fontsize)

ax.fill_between([20000, 25000], 1e-7, 1e-3, linewidth=0, color='k', alpha=0.1)

ax = uf.logticks(ax)
ax = uf.format_axes(ax, labelsize=fontsize)


# plot the actual and predicted spectra in the RH pane
ax = axes[1]
ax.plot(measured_spectrum.index.values, measured_spectrum.values, 'b-', linewidth=linewidth, label='measured')
ax.fill_between(measured_spectrum.index.values, measured_spectrum.values, color='b', linewidth=0, alpha=0.1)
ax.plot(S1_spectrum.index.values, S1_spectrum.values, color='darkred', linestyle='--', linewidth=linewidth, label='predicted S{0}\ncontribution'.format(r'$_1$'))
ax.fill_between(S1_spectrum.index.values, S1_spectrum.values, color='darkred', linewidth=0, alpha=0.1)

ax.set_xlim([545, 805])
ax.set_ylim([0, 1.15])
ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
ax.set_ylabel('PL intensity (norm.)', fontsize=fontsize)
ax.legend(loc='upper right', frameon=False, fontsize=lfontsize)

ax.text(650, 0.6, r'{0} cm$^{{-3}}; 20-25\mu$s'.format(uf.fsci(float(1e19))), fontsize=fontsize)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.2, yminorsep=0.1)
ax = uf.format_axes(ax, labelsize=fontsize)

fig.savefig('figS24_rest.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')