import os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
import matplotlib.colors as mcolors
import usefulfunctions as uf

here = os.path.abspath(os.path.dirname(__file__))

# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 300
linewidth = 1.5
alpha = 0.1


spectrumS1 = pd.read_csv(os.path.join(here, 'data', 'spectrumS1.csv'), index_col=0, header=None, squeeze=True)
spectrumSF = pd.read_csv(os.path.join(here, 'data', 'spectrumSF.csv'), index_col=0, header=None, squeeze=True)
spectrumTTA = pd.read_csv(os.path.join(here, 'data', 'spectrumTTA.csv'), index_col=0, header=None, squeeze=True)
integrated = pd.read_csv(os.path.join(here, 'data', 'fluencedep.csv'), index_col=0, header=0)
    

fig, axes = plt.subplots(nrows=1, ncols=2, gridspec_kw={'wspace': 0.3}, figsize=(14, 5))


# LH panel is spectra
ax = axes[0]
ax.plot(spectrumSF.index, spectrumSF.values, color='blue', linewidth=linewidth, label=r'diF-TES-ADT; $\lambda_{ex}=532$nm', zorder=2)
ax.plot(spectrumTTA.index, spectrumTTA.values, color='seagreen', linewidth=2, alpha=0.6, label=r'diF-TES-ADT:PdPc; $\lambda_{ex}=725$nm', zorder=3)
ax.plot(spectrumS1.index, spectrumS1.values, color='darkred', linewidth=linewidth, linestyle='--', label='_nolegend_', zorder=1)
ax.fill_between(spectrumS1.index, spectrumS1.values, color='darkred', linewidth=0, alpha=0.15)

ax.legend(frameon=False, loc='upper right', fontsize=lfontsize, borderpad=0)

ax.text(710, 0.35, r'20-25$\mu$s (TTA)', color='blue', fontsize=lfontsize)
ax.text(650, 0.75, '200-400ns (TTA)', color='seagreen', fontsize=lfontsize)
ax.text(620, 0.19, r'S$_1$ spectrum', color='darkred', fontsize=lfontsize)

ax.set_xlim([545, 805])
ax.set_ylim([0, 1.4])

ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
ax.set_ylabel('PL Intensity (norm.)', fontsize=fontsize)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.2, yminorsep=0.1)
ax = uf.format_axes(ax, labelsize=fontsize)


# RH panel is fluence dependence
ax = axes[1]
ax.errorbar(integrated.index.values, integrated['pure'].values, yerr=integrated['pure err'].values, color='darkorange', marker='s', linestyle='None', markersize=6, alpha=0.7, capsize=6, label=r'diF-TES-ADT; $\lambda_{ex}=725$nm')
ax.errorbar(integrated.index.values, integrated['doped'].values, yerr=integrated['doped err'].values, color='seagreen', marker='o', linestyle='None', markersize=6, alpha=0.7, capsize=6, label=r'diF-TES-ADT:PdPc; $\lambda_{ex}=725$nm', zorder=3)

ax.plot(integrated.index.values, 0.04*integrated.index.values**2, 'k', linewidth=linewidth, alpha=0.6, label='_nolegend', zorder=2)

ax.set_xscale('log')
ax.set_yscale('log')

lims = list(ax.get_xbound())
ax.fill_between(lims, 5, 1e2, color='black', alpha=0.13, linewidth=0, zorder=1)

ax.text(200, 5e3, 'slope 2', fontsize=fontsize, alpha=0.7, rotation=30)
ax.text(600, 5e1, 'noise', fontsize=fontsize, alpha=0.7)

ax.set_ylim([5, 5e5])
ax.set_xlim(lims)

ax.set_xlabel(r'Pulse energy ($\mu$Jcm$^{{-2}}$)', fontsize=fontsize)
ax.set_ylabel('Integrated PL (counts)', fontsize=fontsize)

ax.legend(frameon=False, loc='upper left', fontsize=lfontsize, borderpad=0)

ax = uf.logticks(ax)
ax = uf.format_axes(ax, labelsize=fontsize)

    
fig.savefig('figS25_all.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')
    