import os
import pandas as pd
import matplotlib.pyplot as plt
import usefulfunctions as uf
from matplotlib.legend_handler import HandlerTuple

here = os.path.abspath(os.path.dirname(__file__))

# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 300
linewidth = 1.5
alpha = 0.1


# load data for panel A - kinetics
film_kinetics = pd.read_csv(os.path.join(here, 'data', 'film_kinetics.csv'), index_col=0, header=None)
kinetics = pd.read_csv(os.path.join(here, 'data', 'crystal_kinetics.csv'), index_col=0, header=None, squeeze=True)
oldsim = pd.read_csv(os.path.join(here, 'data', 'kinetics_simulation_old_rates.csv'), index_col=0, header=None, squeeze=True)
newsim = pd.read_csv(os.path.join(here, 'data', 'kinetics_simulation_new_rates.csv'), index_col=0, header=None, squeeze=True)

# load data for panel B - rate constant variation
rates = pd.read_csv(os.path.join(here, 'data', 'kinetics_rate_constant_modification.csv'), index_col=0, header=None, squeeze=True)

# load data for panel C, D - magnetic field effect
mfe = pd.read_csv(os.path.join(here, 'data', 'mfe.csv'), header=0, index_col=0)
mfe_errors = pd.read_csv(os.path.join(here, 'data', 'mfe_errors.csv'), header=0, index_col=0)
mfe_sim_20_30 = pd.read_csv(os.path.join(here, 'data', 'simulations_20_30.csv'), header=0, index_col=0)
mfe_sim_100_200 = pd.read_csv(os.path.join(here, 'data', 'simulations_100_200.csv'), header=0, index_col=0)



# create the plotting grid
fig, axes = plt.subplots(nrows=2, ncols=2, gridspec_kw={'wspace': 0.2, 'hspace': 0.25}, figsize=(14, 9))


# panel A
ax = axes[0, 0]
ax.loglog(kinetics.index.values, kinetics.values, marker='s', linestyle='None', color='red', mfc='None', mew=1, ms=6, label='crystal (MFE)')
ax.loglog(film_kinetics.index.values, film_kinetics.values, marker='o', linestyle='None', color='lime', mfc='None', mew=1, ms=6, label='film')

ax.loglog(oldsim.index.values, oldsim.values, color='black', alpha=0.6, linestyle='--', linewidth=linewidth, label=r'$k_{tnr}=0$, rest unchanged')
ax.loglog(newsim.index.values, newsim.values, color='black', alpha=0.6, linestyle='-', linewidth=linewidth, label='all rates modified (see b)')

ax.set_xlim([1, 1e5])
ax.set_ylim([1e-5, 2])

ax.set_xlabel('Time (ns)', fontsize=fontsize)
ax.set_ylabel('Integrated PL (norm.)', fontsize=fontsize)

handles, labels = ax.get_legend_handles_labels()
leg1 = ax.legend(handles=handles[0:2], labels=labels[0:2], frameon=False, fontsize=lfontsize, loc='lower left', borderpad=0, handlelength=1.5, handletextpad=0.5)
ax.add_artist(leg1)
leg2 = ax.legend(handles=handles[2:], labels=labels[2:], frameon=False, fontsize=lfontsize, loc='upper right', borderpad=0, handlelength=1.5, handletextpad=0.5)

ax = uf.logticks(ax)
ax = uf.format_axes(ax, labelsize=fontsize)


# panel B
ax = axes[0, 1]
ax.axhline(0, color='0.5', linewidth=1)
ax.bar([r'$k_{{{0}}}$'.format(x[1:].replace('_', '-').lower()) for x in rates.index.values], rates.values, color='dodgerblue')
ax.set_ylabel('variation (%)', fontsize=fontsize, labelpad=-10)
ax.set_ylim([-100, 100])

ax = uf.linearticks(ax, ymajorsep=50, yminorsep=10)
ax = uf.format_axes(ax, labelsize=fontsize)


# panel C
color1 = 'dodgerblue'
color2 = 'darkorange'
factor = 1
ax = axes[1, 0]
ax.errorbar(mfe.index.values, 100*mfe['20-30ns'].values, yerr=100*mfe_errors['20-30ns'].values, capsize=5, color=color1, marker='o', linestyle='None', markersize=6, mew=1, label='20-30ns')
ax.plot(1000*mfe_sim_20_30.index.values, 100*factor*mfe_sim_20_30['8'].values, color=color1, linestyle='-', linewidth=linewidth, label='simulation')

ax.errorbar(mfe.index.values, 100*mfe['100-200ns'].values, yerr=100*mfe_errors['100-200ns'].values, capsize=5, color=color2, marker='s', linestyle='None', markersize=6, mew=1, label='100-200ns')
ax.plot(1000*mfe_sim_100_200.index.values, 100*factor*mfe_sim_100_200['8'].values, color=color2, linestyle='--', linewidth=linewidth, label='simulation')

ax.axhline(0, color='k', alpha=0.5, linewidth=0.5)

ax.set_xlim([0, 260])

ax.set_xlabel('Magnetic Field Strength (mT)', fontsize=fontsize)
ax.set_ylabel(r'$\Delta$PL/PL (%)', fontsize=fontsize)

ax.legend(frameon=False, fontsize=lfontsize, loc=(0.17, 0), ncol=2, columnspacing=0.7)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=10, yminorsep=5)
ax = uf.format_axes(ax, labelsize=fontsize)


# panel D
ax = axes[1, 1]
sigmascale = 4
# 20-30
mean = mfe_sim_20_30.mean(axis=1)
std = mfe_sim_20_30.std(axis=1)
ax.plot(1000*mfe_sim_20_30.index.values, 100*mean.values, color=color1, linestyle=':', linewidth=linewidth, label='mean')
ax.fill_between(1000*mfe_sim_20_30.index.values, 100*(mean.values-4*std.values), 100*(mean.values+4*std.values), color=color1, alpha=0.2, linewidth=0, label=r'$4\sigma$')
ax.plot(1000*mfe_sim_20_30.index.values, 100*factor*mfe_sim_20_30['8'].values, color=color1, linestyle='-', linewidth=linewidth, label=r'$\phi=144^{\circ}$')

#100-200
mean = mfe_sim_100_200.mean(axis=1)
std = mfe_sim_100_200.std(axis=1)
ax.plot(1000*mfe_sim_100_200.index.values, 100*mean.values, color=color2, linestyle=':', linewidth=linewidth, label='mean')
ax.fill_between(1000*mfe_sim_100_200.index.values, 100*(mean.values-4*std.values), 100*(mean.values+4*std.values), color=color2, alpha=0.2, linewidth=0, label=r'$4\sigma$')
ax.plot(1000*mfe_sim_100_200.index.values, 100*factor*mfe_sim_100_200['8'].values, color=color2, linestyle='--', linewidth=linewidth, label=r'$\phi=144^{\circ}$')

ax.axhline(0, xmax=0.64, color='k', alpha=0.5, linewidth=0.5)

ax.set_xlim([0, 260])

ax.set_xlabel('Magnetic Field Strength (mT)', fontsize=fontsize)
ax.set_ylabel(r'$\Delta$PL/PL (%)', fontsize=fontsize)


ax.text(31, 25, '100-200ns', color=color2, fontsize=fontsize)
ax.text(45, -29, '20-30ns', color=color1, fontsize=fontsize)
handles, labels = ax.get_legend_handles_labels()
ax.legend(handles=zip([handles[0], handles[4], handles[1]], [handles[2], handles[5], handles[3]]),
          labels=[labels[0], labels[4], labels[1]],
          handler_map={tuple: HandlerTuple(ndivide=None)}, handletextpad=0.3,
          frameon=False, fontsize=lfontsize, loc=(0.65, 0.35))

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=10, yminorsep=5)
ax = uf.format_axes(ax, labelsize=fontsize)


fig.savefig('figS28_all.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')
