import os
import pandas as pd
import matplotlib.pyplot as plt
import usefulfunctions as uf
from matplotlib.legend_handler import HandlerTuple

here = os.path.abspath(os.path.dirname(__file__))

# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 300
linewidth = 1.5
alpha = 0.1


# load data for panel A - kinetics
kinetics = pd.read_csv(os.path.join(here, 'data', 'crystal_kinetics.csv'), index_col=0, header=None, squeeze=True)
newsim = pd.read_csv(os.path.join(here, 'data', 'kinetics_simulation_new_rates.csv'), index_col=0, header=None, squeeze=True)


# load data for panel B - magnetic field effect
mfe = pd.read_csv(os.path.join(here, 'data', 'mfe.csv'), header=0, index_col=0)
mfe_errors = pd.read_csv(os.path.join(here, 'data', 'mfe_errors.csv'), header=0, index_col=0)
mfe_sim_20_30 = pd.read_csv(os.path.join(here, 'data', 'simulations_20_30.csv'), header=0, index_col=0)
mfe_sim_100_200 = pd.read_csv(os.path.join(here, 'data', 'simulations_100_200.csv'), header=0, index_col=0)


# create the plotting grid
fig, axes = plt.subplots(nrows=1, ncols=2, gridspec_kw={'wspace': 0.2}, figsize=(14, 4.5))


# panel A
ax = axes[0]
ax.loglog(kinetics.index.values, kinetics.values, marker='s', linestyle='None', color='red', mfc='None', mew=1, ms=6, label='measured \'singlet\' PL')
ax.loglog(newsim.index.values, newsim.values, color='black', alpha=0.6, linestyle='-', linewidth=linewidth, label='Merrifield model')

ax.set_xlim([1, 1e5])
ax.set_ylim([1e-5, 2])

ax.set_xlabel('Time (ns)', fontsize=fontsize)
ax.set_ylabel('Integrated PL (norm.)', fontsize=fontsize)

ax.legend(frameon=False, fontsize=lfontsize, loc='upper right')

ax = uf.logticks(ax)
ax = uf.format_axes(ax, labelsize=fontsize)


# panel B
color1 = 'dodgerblue'
color2 = 'darkorange'
factor = 1

ax = axes[1]
sigmascale = 4
# 20-30
mean = mfe_sim_20_30.mean(axis=1)
std = mfe_sim_20_30.std(axis=1)

ax.errorbar(mfe.index.values, 100*mfe['20-30ns'].values, yerr=100*mfe_errors['20-30ns'].values, capsize=5, color=color1, marker='o', linestyle='None', markersize=6, mew=1, label='20-30ns')

ax.plot(1000*mfe_sim_20_30.index.values, 100*mean.values, color=color1, linestyle=':', linewidth=linewidth, label='mean')
ax.fill_between(1000*mfe_sim_20_30.index.values, 100*(mean.values-4*std.values), 100*(mean.values+4*std.values), color=color1, alpha=0.2, linewidth=0, label=r'$4\sigma$')
#ax.plot(1000*mfe_sim_20_30.index.values, 100*factor*mfe_sim_20_30['8'].values, color=color1, linestyle='-', linewidth=linewidth, label=r'$\phi=144^{\circ}$')

#100-200
mean = mfe_sim_100_200.mean(axis=1)
std = mfe_sim_100_200.std(axis=1)

ax.errorbar(mfe.index.values, 100*mfe['100-200ns'].values, yerr=100*mfe_errors['100-200ns'].values, capsize=5, color=color2, marker='s', linestyle='None', markersize=6, mew=1, label='100-200ns')

ax.plot(1000*mfe_sim_100_200.index.values, 100*mean.values, color=color2, linestyle=':', linewidth=linewidth, label='mean')
ax.fill_between(1000*mfe_sim_100_200.index.values, 100*(mean.values-4*std.values), 100*(mean.values+4*std.values), color=color2, alpha=0.2, linewidth=0, label=r'$4\sigma$')
#ax.plot(1000*mfe_sim_100_200.index.values, 100*factor*mfe_sim_100_200['8'].values, color=color2, linestyle='--', linewidth=linewidth, label=r'$\phi=144^{\circ}$')

ax.axhline(0, color='k', alpha=0.5, linewidth=0.5)

ax.set_xlim([0, 260])
ax.set_ylim([-18, 14])

ax.set_xlabel('Magnetic Field Strength (mT)', fontsize=fontsize)
ax.set_ylabel(r'$\Delta$PL/PL (%)', fontsize=fontsize, labelpad=-10)

ax.text(170, -4, '100-200ns', color=color2, fontsize=fontsize)
ax.text(70, 9, '20-30ns', color=color1, fontsize=fontsize)
handles, labels = ax.get_legend_handles_labels()
ax.legend(handles=zip([handles[0], handles[4], handles[1]], [handles[2], handles[5], handles[3]]),
          labels=[labels[0], labels[4], labels[1]],
          handler_map={tuple: HandlerTuple(ndivide=None)}, handletextpad=0.3,
          frameon=False, fontsize=lfontsize, loc=(0.38, 0), borderpad=0.1)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=10, yminorsep=5)
ax = uf.format_axes(ax, labelsize=fontsize)

fig.savefig('figS30_all.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')
