import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf

here = os.path.abspath(os.path.dirname(__file__))


# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 150
linewidth = 1
alpha = 0.1


# load diF-TES-ADT absorption and PL
dt_abs = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_abs.csv'), index_col=0, header=0)
dt_abs = dt_abs[(dt_abs.index >= 390) & (dt_abs.index <= 710)]
dt_pl = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_pl.csv'), index_col=0, header=0)
dt_pl = dt_pl[(dt_pl.index >= 510) & (dt_pl.index <= 790)]

# make the plotting grid
fig, axes = plt.subplots(nrows=2, ncols=5,
                         gridspec_kw={'hspace': 0.3,
                                      'wspace': 0.15,
                                      'width_ratios': [1, 0.05, 0.3, 1, 0.05]},
                         figsize=(14, 8.5))


# colourmap of absorption
ax = axes[0, 0]
dt_abs_norm = dt_abs.apply(lambda x: x/x.max())
x = dt_abs_norm.index.values
y = np.array([int(x.replace('K', '')) for x in dt_abs_norm.columns])
X, Y = np.meshgrid(x, y)
Z = np.transpose(dt_abs_norm.values)
cs = ax.contourf(X, Y, Z, levels=np.linspace(0, 1, 51), cmap=plt.cm.inferno, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
ax.set_rasterization_zorder(-10)
cbar = fig.colorbar(cs, cax=axes[0, 1], ticks=(0, 0.5, 1), aspect=20)
cbar.ax.tick_params(axis='y', which='major', labelsize=fontsize, width=1.4, length=6)
cbar.ax.set_ylabel('Absorbance (norm.)', fontsize=fontsize)
ax.set_ylabel('Temperature (K)', fontsize=fontsize)
ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
ax = uf.format_axes(ax, labelsize=fontsize)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=50, yminorsep=10)


# colourmap of pl
ax = axes[0, 3]
dt_pl_norm = dt_pl.apply(lambda x: x/x.max())
x = dt_pl_norm.index.values
y = np.array([int(x.replace('K', '')) for x in dt_pl_norm.columns])
X, Y = np.meshgrid(x, y)
Z = np.transpose(dt_pl_norm.values)
cs = ax.contourf(X, Y, Z, levels=np.linspace(0, 1, 51), cmap=plt.cm.inferno, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
ax.set_rasterization_zorder(-10)
cbar = fig.colorbar(cs, cax=axes[0, 4], ticks=(0, 0.5, 1))
cbar.ax.set_ylabel('PL intensity (norm.)', fontsize=fontsize)
cbar.ax.tick_params(axis='y', which='major', labelsize=fontsize, width=1.4, length=6)
ax.set_ylabel('Temperature (K)', fontsize=fontsize)
ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
ax = uf.format_axes(ax, labelsize=fontsize)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=50, yminorsep=10)

# absorption lines
dt_abs_to75 = dt_abs[['75K', '100K', '125K', '150K', '175K', '200K', '225K', '250K', '275K', '300K']]
ax = axes[1, 0]
for i, temperature in enumerate(dt_abs_to75.columns):
    ax.plot(dt_abs_to75.index.values, dt_abs_to75[temperature].values, color=plt.cm.coolwarm(int(255*(i/ len(dt_abs_to75.columns)))), alpha=1, linewidth=linewidth, label=temperature)
ax.set_xlim([390, 710])
ax.set_ylim([0, 0.27])
ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
ax.set_ylabel('Absorbance', fontsize=fontsize)
ax.legend(frameon=False, loc='upper right', fontsize=12, borderpad=0)
ax = uf.format_axes(ax, labelsize=fontsize)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.1, yminorsep=0.02)
        
# pl lines
dt_pl_to75 = dt_pl[['75K', '100K', '125K', '150K', '175K', '200K', '225K', '250K', '275K', '300K']]/1e4
ax = axes[1, 3]
for i, temperature in enumerate(dt_pl_to75.columns):
    ax.plot(dt_pl_to75.index.values, dt_pl_to75[temperature].values, color=plt.cm.coolwarm(int(255*(i/ len(dt_pl_to75.columns)))), alpha=1, linewidth=linewidth, label=temperature)
ax.set_xlim([510, 790])
ax.set_ylim([0, 7.6])
ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
ax.set_ylabel(r'PL intensity (10$^4$ counts)', fontsize=fontsize) 
ax.legend(frameon=False, loc='upper right', fontsize=12, borderpad=0)
ax = uf.format_axes(ax, labelsize=fontsize)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=2, yminorsep=0.5)

axes[0, 2].axis('off')
axes[1, 1].axis('off')
axes[1, 2].axis('off')
axes[1, 4].axis('off')

fig.savefig('figS6_abcd.{0}'.format(fmt), format=fmt, dpi=dpi, bbox_inches='tight')







