import os
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
import matplotlib.colors as mcolors
import usefulfunctions as uf

here = os.path.abspath(os.path.dirname(__file__))

# plot parameters
fontsize = 16
lfontsize = 16
fullpagewidth = 7
fmt = 'svg'
dpi = 150
linewidth = 1.5
alpha = 0.1


# load all the data into a dict
temp = '77K'
fignum = 'S7'
last_time = 3e4
N0s = [1e17, 1e18, 1e19]
data = {}
for N0 in N0s:
    data[N0] = {}
    # TRPL
    trpl = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_{1}cm3_TRPL.csv'.format(temp, str(N0).replace('+', ''))), header=0, index_col=0)
    trpl.columns = [int(x.replace('.1', '')) for x in trpl.columns]
    data[N0]['TRPL'] = trpl.loc[:, trpl.columns[trpl.columns <= last_time]]
    # MCR-ALS spectra
    spectra = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_{1}cm3_MCR-ALS_spectra.csv'.format(temp, str(N0).replace('+', ''))), header=0, index_col=0)
    data[N0]['MCR-ALS spectra'] = spectra
    # MCR-ALS kinetics
    kinetics = pd.read_csv(os.path.join(here, 'data', 'diF-TES-ADT_{0}_{1}cm3_MCR-ALS_kinetics.csv'.format(temp, str(N0).replace('+', ''))), header=0, index_col=0)
    data[N0]['MCR-ALS kinetics'] = kinetics[kinetics.index <= last_time]
    
    
# create plotting grid
fig, axes = plt.subplots(nrows=4,
                         ncols=3,
                         gridspec_kw={'hspace': 0.1,
                                      'wspace': 0.45,
                                      'height_ratios': [0.05, 1, 1, 1],
                                      'width_ratios': [1.0, 1.0, 1.0]},
                         figsize=(14, 11))

axes[0, 1].axis('off')
axes[0, 2].axis('off')


# plot
for row, N0 in enumerate(N0s):
    
    row += 1
    
    # colourmap in col 1
    ax = axes[row, 0]
    trpl = data[N0]['TRPL']
    trpl[trpl < 1e-7] = 1e-7
    x = trpl.index.values
    y = trpl.columns.values
    X, Y = np.meshgrid(x, y)
    Z = np.transpose(trpl.values)
    levels = np.logspace(-7, 0, 51)
    cs = ax.contourf(X, Y, Z, levels, norm=mcolors.LogNorm(), cmap=plt.cm.inferno, extend='both', zorder=-20)
    ax.set_yscale('log')
    for c in cs.collections:
        c.set_edgecolor('face')
        ax.set_rasterization_zorder(-10)
    
    if row == 1:
        cbar = fig.colorbar(cs, cax=axes[0, 0], ticks=np.logspace(-6, 0, 4), orientation='horizontal')
        cbar.ax.set_xlabel('PL intensity (norm.)', fontsize=fontsize, labelpad=-60)
        cbar.ax.tick_params(axis='x', which='major', labelsize=fontsize, width=1.4, length=6)
        axes[0, 0].xaxis.set_ticks_position('top')
        ax.set_xticklabels([])
    elif row == 2:
        ax.set_xticklabels([])
    elif row == 3:
        ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
        
    ax.set_ylabel('Time (ns)', fontsize=fontsize)
    ax.set_xlim([545, 805])
    ax.set_ylim([1, max(y)])
    
    ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)
    ax = uf.logticks(ax, which='y')
    ax = uf.format_axes(ax, labelsize=fontsize)

    ax.text(0.53, 0.88, r'{0} cm$^{{-3}}$'.format(uf.fsci(float(N0))), transform=ax.transAxes, color='w', fontsize=fontsize)
    
    # MCR-ALS spectra in col 2
    ax = axes[row, 1]
    spectra = data[N0]['MCR-ALS spectra']
    ax.plot(spectra.index.values, spectra['S1'].values, color='darkred', label=r'S$_1$', linewidth=linewidth)
    ax.fill_between(spectra.index.values, spectra['S1'].values, color='darkred', alpha=0.1, linewidth=0)
    ax.plot(spectra.index.values, spectra['TT'].values, color='blue', label=r'$^1$(TT)', linewidth=linewidth)
    ax.fill_between(spectra.index.values, spectra['TT'].values, color='blue', alpha=0.2, linewidth=0) 
    ax.set_xlim([545, 805])
    ax.set_ylim([0, 1.15])
    ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
    ax.set_ylabel('PL Intensity (norm.)', fontsize=fontsize)
    ax.legend(loc='upper right', frameon=False, fontsize=lfontsize, borderpad=0, handlelength=1.5)
    
    if row == 1:
        ax.set_title('MCR-ALS Spectra', fontsize=fontsize, pad=10)
        ax.set_xticklabels([])
    elif row == 2:
        ax.set_xticklabels([])
    elif row == 3:
        ax.set_xlabel('Wavelength (nm)', fontsize=fontsize)
        
    ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.2, yminorsep=0.1)
    ax = uf.format_axes(ax, labelsize=fontsize)
    
    # MCR-ALS kinetics in col 3
    ax = axes[row, 2]
    kinetics = data[N0]['MCR-ALS kinetics']
    ax.loglog(kinetics.index.values, kinetics['S1'].values, markersize=6, marker='o', color='darkred', linestyle='None', mfc='None', markeredgewidth=1, label=r'S$_1$')
    ax.loglog(kinetics.index.values, kinetics['TT'].values, markersize=6, marker='o', color='blue', linestyle='None', mfc='None', markeredgewidth=1, label=r'$^1$(TT)')
    ax.yaxis.set_major_locator(mticker.LogLocator(numticks=12))
    ax.yaxis.set_minor_locator(mticker.LogLocator(subs=np.linspace(0.1, 0.9, 9), numticks=12))
    ax.xaxis.set_major_locator(mticker.LogLocator(numticks=12))
    ax.xaxis.set_minor_locator(mticker.LogLocator(subs=np.linspace(0.1, 0.9, 9), numticks=12))
    ax.set_xlim([1, 1e5])
    ax.set_ylim([1e-7, 2])
    ax.set_xlabel('Time (ns)', fontsize=fontsize)
    ax.set_ylabel('PL Intensity (norm.)', fontsize=fontsize)
    ax.legend(loc='upper right', fontsize=lfontsize, framealpha=0.8, edgecolor='None', borderpad=0, handlelength=0.5)
    
    if row == 1:
        ax.set_title('MCR-ALS Dynamics', fontsize=fontsize, pad=10)
        ax.set_xticklabels([])
    elif row == 2:
        ax.set_xticklabels([])
    elif row == 3:
        ax.set_xlabel('Time (ns)', fontsize=fontsize)
        
    ax = uf.logticks(ax)
    ax = uf.format_axes(ax, labelsize=fontsize)
    
fig.savefig('fig{1}_all.{0}'.format(fmt, fignum), format=fmt, dpi=dpi, bbox_inches='tight')
    